import 'dart:async';

import 'package:dcoupon_sdk_flutter_plugin_example/translations.dart';
import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:dcoupon_sdk_flutter_plugin/dcoupon_sdk_flutter_plugin.dart';
import 'constants.dart';

void main() => runApp(ExampleApp());

class ExampleApp extends StatelessWidget {
// This widget is the root of your example application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        localizationsDelegates: [
          const TranslationsDelegate(),
          GlobalMaterialLocalizations.delegate,
          GlobalWidgetsLocalizations.delegate,
        ],
        supportedLocales: [
          const Locale('en'),
          const Locale('es'),
        ],
        home: HomePage());
  }
}

class HomePage extends StatefulWidget {
  HomePage({Key key}) : super(key: key);

  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<HomePage> {
  _LoginPageState();

  final email = TextEditingController();
  final password = TextEditingController();

  @override
  initState() {
    super.initState();
  }

  Future<void> startTest() async {
    try {
      print("DART.ExampleApp.startTest()");
      DcouponSdkFlutterPlugin dcouponSdkFlutterPlugin = new DcouponSdkFlutterPlugin();
      final String result = await dcouponSdkFlutterPlugin.invokeMethod('startTest', {
        Constants.EMAIL: email.text,
        Constants.EXTERNAL_ID: 'fromDartToAndroid',
        Constants.ALIAS: 'flutterAlias',
        Constants.GENDER: 'FEMALE',
        Constants.BIRTHDATE: '1991-04-12'
      });
      print("DART.startTest().result -> " + result.toString());
    } on Exception catch (e) {
      print("DART.startTest().Failed: " + e.toString());
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(Translations.of(context).text('example_home_page_name')),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            new Image(
                image: new AssetImage('lib/assets/images/dcoupon_logo.png')),
            TextFormField(
                controller: email,
                decoration: InputDecoration(
                    hintText:
                        Translations.of(context).text('example_text_hint'),
                    labelText:
                        Translations.of(context).text('example_label_email'))),
            TextFormField(
              controller: password,
              decoration: InputDecoration(
                  hintText: Translations.of(context).text('example_text_hint'),
                  labelText:
                      Translations.of(context).text('example_label_password')),
            ),
            RaisedButton(
              onPressed: startTest,
              child: Text(
                  Translations.of(context).text('example_button_start_test'),
                  style: TextStyle(fontSize: 20)),
            )
          ],
        ),
      ),
    );
  }
}
