import 'dart:async';

import 'package:flutter/services.dart';

import 'constants.dart';
import 'httpenrollmentform.dart';
import 'httplogin.dart';
import 'httprequest.dart';

class DcouponSdkFlutterPlugin {
  static const MethodChannel _channel =
      const MethodChannel('dcoupon_sdk_flutter_plugin');
  static const String _urlThirdPartyAPI =
      "https://0me53fpvz5.execute-api.eu-west-1.amazonaws.com/test/";
  DcouponSdkFlutterPlugin(){
    _channel.setMethodCallHandler(_methodCallHandler);
  }
  Future<dynamic> invokeMethod(String method, dynamic args) async{
    return await _channel.invokeMethod(method, args);
  }
  Future<dynamic> _methodCallHandler(MethodCall call) async {
    var completer = new Completer();
    print("DART.DcouponSdkFlutterPlugin._methodCallHandler().method -> " + call.method);
    // stream controller to handle the http response
    var streamController = StreamController<HttpResponse>();
    streamController.stream.listen((data) {
      Map<String, dynamic> map = new Map();
      map.putIfAbsent(
          HttpInterface.STATUS_CODE, () => data.statusCode.toString());
      map.putIfAbsent(HttpInterface.BODY, () => data.body.toString());
      map.putIfAbsent(HttpInterface.HEADERS, () => data.headers.toString());
      completer.complete(map);
    });
    switch (call.method) {
      case Constants.LOGIN:
        HttpLogin httpLogin = new HttpLogin(
            call.arguments[Constants.EMAIL],
            call.arguments[Constants.CLIENT_API_TOKEN],
            call.arguments[Constants.EXTERNAL_ID],
            call.arguments[Constants.ALIAS],
            call.arguments[Constants.GENDER],
            call.arguments[Constants.BIRTHDATE],
            _urlThirdPartyAPI + Constants.PATH_LOGIN);
        httpLogin.setStreamController(streamController);
        await httpLogin.execute();
        break;
      case Constants.ENROLLMENT_FORM:
        HttpEnrollmentForm httpEnrollmentForm = new HttpEnrollmentForm(
            call.arguments[Constants.CLIENT_API_TOKEN],
            call.arguments[Constants.URL_REDIRECT]);
        httpEnrollmentForm.setStreamController(streamController);
        await httpEnrollmentForm.execute();
        break;
      case Constants.ACCEPT_ENROLLMENT:

        break;
      default:
        HttpResponse httpResponse = new HttpResponse();
        httpResponse.body =
        "---------------------- not implemented (${call.method}) ----------------------------";
        httpResponse.statusCode = HttpInterface.HTTP_NOT_FOUND;
        streamController.sink.add(httpResponse);
        print(
            "---------------------- not implemented (${call.method}) ----------------------------");
        break;
    }
    return completer.future;
  }
}
