import 'dart:convert';
import 'dart:io';

abstract class HttpInterface {
  static const String POST = "POST";
  static const String GET = "GET";
  static const String STATUS_CODE = "statusCode";
  static const String RESPONSE_CODE = "responseCode";
  static const String HEADERS = "headers";
  static const String BODY = "body";
  static const String RESPONSE = "response";
  static const String HTTP_MESSAGE = "http-message";
  static const String RESPONSE_MSG = "responseMsg";
  static const String MESSAGE = "message";
  static const String SESSION_TOKEN = "sessionToken";
  static const String CLIENT_TYPE = "client_type";
  static const String LOCATION = "location";
  static const String UTF_8 = "UTF-8";
  static const int HTTP_INTERNAL_ERROR = 500;
  static const int HTTP_UNAVAILABLE = 503;
  static const int HTTP_UNAUTHORIZED = 401;
  static const int HTTP_NOT_FOUND = 404;
  static const int HTTP_MOVED_PERM = 301;
  static const int HTTP_MOVED_TEMP = 302;
  static const int HTTP_OK = 200;
}

abstract class HttpRequest extends HttpInterface {
  String url, method;
  Map<String, String> properties = new Map();
  Map<String, String> payload = new Map();

  Future execute() async {
    onPreExecute();
    var httpClient = HttpClient();
    HttpResponse httpResponse = new HttpResponse();
    if (getMethod() == HttpInterface.POST) {
      await httpClient
          .postUrl(Uri.parse(getUrl()))
          .then((HttpClientRequest request) {
        // Optionally set up headers...
        // Optionally write to the request object...
        // Then call close.
        properties.forEach((key, value) {
          request.headers.add(key, value);
        });
        if (payload.length > 0) {
          request.write(json.encode(payload).toString());
        }
        return request.close();
      }).then((HttpClientResponse response) {
        // Process the response.
        httpResponse.statusCode = response.statusCode;
        response.headers.forEach((key, value) {
          httpResponse.headers.putIfAbsent(key, () => value[0].toString());
        });
        response.transform(utf8.decoder).listen((contents) {
          // handle data
          httpResponse.body = contents;
          onPostExecute(httpResponse);
        });
      });
    } else if (getMethod() == HttpInterface.GET) {
      await httpClient
          .getUrl(Uri.parse(getUrl()))
          .then((HttpClientRequest request) {
        // Optionally set up headers...
        // Optionally write to the request object...
        // Then call close.
        properties.forEach((key, value) {
          request.headers.add(key, value);
        });
        return request.close();
      }).then((HttpClientResponse response) {
        // Process the response.
        httpResponse.statusCode = response.statusCode;
        response.headers.forEach((key, value) {
          httpResponse.headers.putIfAbsent(key, () => value[0].toString());
        });
        response.transform(utf8.decoder).listen((contents) {
          // handle data
          httpResponse.body = contents;
          onPostExecute(httpResponse);
        });
      });
    } else
      throw Exception;
    httpClient.close();
  }

  String getUrl() {
    return url;
  }

  void setUrl(String url) {
    this.url = url;
  }

  String getMethod() {
    return method;
  }

  void setMethod(String method) {
    this.method = method;
  }

  void onPreExecute();

  void onPostExecute(HttpResponse httpResponse);
}

class HttpResponse {
  int statusCode;
  Map<String, String> headers = new Map();
  String body;
}
